/*
    HARMONIC - Online music communities system
    ========
    
    Copyright (C) 2008  Juan Pedro Bolivar Puente,
                        Luca Mefistofeles Conesa Martin-Aragon

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package harmonic;

import java.util.Map;

public class Singleton<T>
{
    private static Map<Class<?>,Object> instanceMap =
        new java.util.HashMap<Class<?>,Object>();

    public static synchronized <T>
	T instance(Class<T> clazz)
	    throws InstantiationException,
	    IllegalAccessException
    {
        T instance = clazz.cast(instanceMap.get(clazz));
        if (instance != null) {
            return instance;
        }
        
        try {
            instance = clazz.cast(clazz.newInstance());
        } catch (InstantiationException exc) {
            throw new IllegalArgumentException(exc);
        } catch (IllegalAccessException exc) {
            throw new IllegalArgumentException(exc);
        } catch (ClassCastException exc) {
            throw new IllegalArgumentException(exc);
        }
        
        instanceMap.put(clazz, instance);
        
        return instance;
    }
}
